# -*-coding:utf-8 -*- #
# ---------------------------------------------------------------------------
# ProjectName:   Software_Testing
# FileName:      read_excel.py
# Author:       Rikka
# Datetime:     2024/7/13 10:32
# Description:
# 命名规范：文件名全小写+下划线，类名大驼峰，方法和变量小写+下划线连接，
# 常量大写，变量和常量用名词，方法用动词
# ---------------------------------------------------------------------------
import openpyxl

from APIAutoTest_v3_1_multi_user import log
from APIAutoTest_v3_1_multi_user.common.read_basic_ini import ReadBasicIni
from APIAutoTest_v3_1_multi_user.common.read_ini import ReadIni
from APIAutoTest_v3_1_multi_user.common.read_json import read_json


class ReadExcel:
    def __init__(self, username):
        """
            1：加载excel的工作簿，再获取工作表
            2: 获取所有json文件的路径，并读取所有的json文件，将json文件的数据赋值给一个变量。
        """
        self.ini = ReadIni(username)
        self.basic_ini = ReadBasicIni()
        case_data_path = self.ini.get_file_path("case")
        expect_data_path = self.ini.get_file_path("expect")
        sql_data_path = self.ini.get_file_path("sql")

        excel_path = self.ini.get_file_path("excel")
        table_name = self.ini.get_table_name("table")

        self.case_data_dict = read_json(case_data_path)
        self.expect_data_dict = read_json(expect_data_path)
        self.sql_data_dict = read_json(sql_data_path)

        try:
            wb = openpyxl.load_workbook(excel_path)
            self.ws = wb[table_name]
        except Exception as e:
            log.error(f"获取excel工作簿或工作表时报错，错误为:{e}")
            raise e

    def __get_cell_value(self, column:str ,row:int) -> str:
        """
        获取指定单元格数据
        :param column: 列号
        :param row: 行号
        :return: str|None
        """
        try:
            cell_value = self.ws[column+str(row)].value
        except Exception as e:
            log.error(f"获取单元格数据时报错，错误为:{e}")
            raise e
        else:
            # 判断单元格数据是否为None，如果是返回None
            if cell_value is None:
                return None
            # 判断单元格数据去掉前后空格之后是否为空的字符串，如果是空的字符串，返回None，如果不是空的字符串返回去掉前后空格的数据
            elif cell_value.strip():
                return cell_value.strip()

    def module_name(self, row):
        """根据行号，获取模块名称"""
        return self.__get_cell_value("b", row)

    def api_name(self, row):
        """根据行号获取接口名称"""
        return self.__get_cell_value("c", row)

    def case_title(self, row):
        """根据行号，获取用例标题"""
        return self.__get_cell_value("d", row)

    def case_level(self, row):
        """根据行号，获取用例等级"""
        return self.__get_cell_value("e", row)

    def req_method(self, row):
        """根据行号，获取用例的请求方法"""
        return self.__get_cell_value("f", row)

    def req_url(self, row):
        """根据行号，获取用例请求的url"""
        host = self.basic_ini.get_url("test_host")
        path = self.__get_cell_value("g", row)
        # 判断用例的请求路径是否为None，如果为None返回None，如果不为None，进行url的拼接
        if path:
            return host + path

    def case_mime(self, row):
        """根据行号，获取用例的媒体类型"""
        mime = self.__get_cell_value("h", row)
        # 判断用例的媒体类型的值是否为None，如果为None，返回None，如果不为None，将媒体类型的值转小写,再返回
        if mime:
            return mime.lower()

    def case_data(self, row):
        """根据行号，获取用例数据"""
        case_data_key = self.__get_cell_value("i", row)
        if case_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            return self.case_data_dict[module_name][api_name][case_data_key]

    def expect_data(self, row):
        """根据行号，获取期望数据"""
        expect_data_key = self.__get_cell_value("j", row)
        if expect_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            return self.expect_data_dict[module_name][api_name][expect_data_key]

    def sql_type(self, row):
        """根据行号获取sql语句的类型"""
        sql_type = self.__get_cell_value("k", row)
        if sql_type:
            return sql_type.lower()

    def sql_data(self, row):
        """根据行号，获取期望数据"""
        sql_data_key = self.__get_cell_value("l", row)
        if sql_data_key:
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            return self.sql_data_dict[module_name][api_name][sql_data_key]

    def update_key(self, row):
        """根据行号，获取用例更新的key"""
        return self.__get_cell_value("m", row)

    def get_data(self):
        data_list = []
        for row in range(2, self.ws.max_row + 1):
            module_name = self.module_name(row)
            api_name = self.api_name(row)
            case_title = self.case_title(row)
            case_level = self.case_level(row)
            case_method = self.req_method(row)
            case_url = self.req_url(row)
            case_mime = self.case_mime(row)
            case_data = self.case_data(row)
            expect_data = self.expect_data(row)
            sql_type = self.sql_type(row)
            sql_data = self.sql_data(row)
            update_key = self.update_key(row)
            # 过滤空行--如果请求方法或请求的路径为None就是空行
            if case_method is not None and case_url is not None:
                data_list.append([module_name, api_name, case_title, case_level, case_method, case_url, case_mime, case_data, expect_data, sql_type, sql_data, update_key])
        else:
            return data_list


if __name__ == '__main__':
    excel = ReadExcel("六花")
    print(excel.get_data())
